@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_common_external_h =
- *   "$Id: external.h 12949 2010-04-25 07:07:43Z mwedel $";
+ *   "$Id: external.h 14014 2010-10-21 06:18:46Z mwedel $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -48,7 +48,7 @@ extern void draw_magic_map(void);
 
 /* Info related functions */
 extern void draw_prompt(const char *str);
-extern void draw_ext_info(int orig_color, int type, int subtype, char *message);
+extern void draw_ext_info(int orig_color, int type, int subtype, const char *message);
 extern void x_set_echo(void);
 extern void set_scroll(const char *s);
 extern void set_autorepeat(const char *s);
@@ -93,14 +93,18 @@ extern void client_pickup(uint32 pickup);
 
 /* Account Login Functions */
 extern void start_login(int method);
-extern void hide_all_login_windows();
+extern void hide_all_login_windows(void);
 extern void account_login_failure(char *message);
 extern void account_creation_failure(char *message);
 extern void account_add_character_failure(char *message);
 extern void create_new_character_failure(char *message);
-extern void choose_character_init();
+extern void choose_character_init(void);
 extern void update_character_choose(const char *name, const char *class,
                              const char *race, const char *face,
                              const char *party, const char *map,
                              int level, int faceno);
 extern void update_login_info(int type);
+
+/* Character Creation Functions */
+extern void new_char_window_update_info();
+extern void starting_map_update_info();
