@@ -1,5 +1,5 @@
 const char * const rcsid_common_metaserver_c =
-    "$Id: metaserver.c 12946 2010-04-24 19:44:18Z cavesomething $";
+    "$Id: metaserver.c 13981 2010-10-09 21:43:20Z anmaster $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -39,12 +39,12 @@ const char * const rcsid_common_metaserver_c =
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <client.h>
 #include <cconfig.h>
 #include <external.h>
 
-#include <pthread.h>
 #include <metaserver.h>
 
 #ifdef HAVE_CURL_CURL_H
@@ -53,16 +53,16 @@ const char * const rcsid_common_metaserver_c =
 #include <curl/easy.h>
 #endif
 
-
 Meta_Info *meta_servers = NULL;
 
 int meta_numservers = 0;
 
 int meta_sort(Meta_Info *m1, Meta_Info *m2) { return strcasecmp(m1->hostname, m2->hostname); }
 
-/* This checks the servers sc_version and cs_version to see
+/**
+ * This checks the servers sc_version and cs_version to see
  * if they are compatible.
- * @parm entry
+ * @param entry
  * entry number in the metaservers array to check.
  * @return
  * 1 if this entry is compatible, 0 if it is not.  Note that this can
@@ -110,11 +110,23 @@ char *cached_servers_ip[CACHED_SERVERS_MAX];
 static int cached_servers_loaded = 0;
 const char *cached_server_file = NULL;
 
-
+/**
+ * Load server names and addresses or DNS names from a cache file found in the
+ * player's client data folder.  The cache file has traditionally been named
+ * "servers.cache".  The server cache file is a plain text file that is
+ * line-feed delimited.  Cache entries consist of two lines each and if the
+ * file has an odd number of lines, the last entry is ignored.  The first
+ * line of a cache entry is the name of the server, and the second line is an
+ * IP address or DNS hostname.  Metaserver uses both entries.  Metaserver 2
+ * uses only the name since most servers set the name to a hostname anyway.
+ * The load function does no parsing, so the entries must be in the correct
+ * order.  There is no mechanism to support comments.  If a file has an odd
+ * number of lines, the loader assumes the last line is an incomplete entry
+ * and silently discards it.
+ */
 static void metaserver_load_cache(void) {
+    char name[MS_LARGE_BUF], ip[MS_LARGE_BUF];
     FILE *cache;
-    char buf[ MS_LARGE_BUF ];
-    int name;
 
     if (cached_servers_loaded || !cached_server_file)
         return;
@@ -127,24 +139,20 @@ static void metaserver_load_cache(void) {
     if (!cache)
         return;
 
-    name = 0;
-    while (fgets(buf, MS_LARGE_BUF, cache) != NULL && cached_servers_num < CACHED_SERVERS_MAX) {
-        buf[strlen(buf)-1] = 0;
-        if (!name) {
-            name = 1;
-            cached_servers_name[cached_servers_num] = strdup(buf);
-        } else {
-            name = 0;
-            cached_servers_ip[cached_servers_num++] = strdup(buf);
-        }
+    while (cached_servers_num < CACHED_SERVERS_MAX
+    &&     fgets(name, MS_LARGE_BUF, cache) != NULL
+    &&     fgets(ip  , MS_LARGE_BUF, cache) != NULL) {
+        ip[strlen(ip)-1] = 0;
+        name[strlen(name)-1] = 0;
+        cached_servers_ip[cached_servers_num] = strdup(ip);
+        cached_servers_name[cached_servers_num++] = strdup(name);
     }
     fclose(cache);
-    if (name) {
-        /* Missing IP? */
-        cached_servers_num--;
-    }
 }
 
+/**
+ *
+ */
 static void metaserver_save_cache(void) {
     FILE *cache;
     int server;
@@ -163,6 +171,76 @@ static void metaserver_save_cache(void) {
     fclose(cache);
 }
 
+/**
+ * Add a server to the players server cache file.
+ * @param server_name
+ * @param server_ip
+ */
+void metaserver_update_cache(const char *server_name, const char *server_ip) {
+    int index;
+
+    /*
+     * Try to find the given server name in the existing server cache.  If the
+     * zero-based index ends up equal to the one-based number of cached
+     * servers, it was not found.
+     */
+    for (index = 0; index < cached_servers_num; index++) {
+        if (strcmp(server_name, cached_servers_name[index]) == 0) {
+            break;
+        }
+    }
+
+    /*
+     * If server is already first in the cache list, nothing else needs to be
+     * done, otherwise, the server needs to be cached.
+     */
+    if (index != 0 || !cached_servers_num) {
+        char *name;
+        char *ip;
+        int  copy;
+
+        if (index == cached_servers_num) {
+            /*
+             * If the server was not found in the cache, expand the cache size
+             * by one unless that creates too many entries.
+             */
+            name = strdup(server_name);
+            ip = strdup(server_ip);
+            cached_servers_num++;
+            if (cached_servers_num > CACHED_SERVERS_MAX) {
+                cached_servers_num--;
+                free(cached_servers_name[cached_servers_num-1]);
+                free(cached_servers_ip[cached_servers_num-1]);
+            }
+        } else {
+            /*
+             * If the server was already listed in the cache, grab a copy of
+             * the prior listing.
+             */
+            name = cached_servers_name[index];
+            ip = cached_servers_ip[index];
+        }
+
+        /*
+         * If the server as already listed, move all the cached items above
+         * the listing down a slot, otherwise, move the whole list down a
+         * notch.  This "empties" the top slot.
+         */
+        for (copy = MIN(index, CACHED_SERVERS_MAX-1); copy > 0; copy--) {
+            cached_servers_name[copy] = cached_servers_name[copy-1];
+            cached_servers_ip[copy] = cached_servers_ip[copy-1];
+        }
+
+        /*
+         * Put the added server information at the top of the cache list, and
+         * save the changes.
+         */
+        cached_servers_name[0] = name;
+        cached_servers_ip[0] = ip;
+        metaserver_save_cache();
+    }
+}
+
 /*****************************************************************************
  * End of cache related functions.
  *****************************************************************************/
@@ -192,7 +270,7 @@ static int ms2_is_running=0;
 static char *metaservers[] = {"http://crossfire.real-time.com/metaserver2/meta_client.php"};
 
 /**
- * Curle doesn't really have any built in way to get data
+ * Curl doesn't really have any built in way to get data
  * from the URL into string data - instead, we get a blob
  * of data which we need to find the newlines, etc
  * from.  Curl also provides the data in multiple calls
@@ -457,8 +535,6 @@ int metaserver2_get_info(void) {
     return 0;
 }
 
-
-
 /**
  * Does single use initalization of metaserver2 variables.
  */
@@ -488,9 +564,10 @@ static int ms1_is_running=0;
 /* Need script.h for script_killall */
 #include <script.h>
 
-/* This gets input from a socket, and returns it one line at a time.
+/**
+ * This gets input from a socket, and returns it one line at a time.
+ * This is a Windows-specific function, since you can't use fgets under Win32
  */
-/* This is a Windows-specific function, since you can't use fgets under Win32 */
 char *get_line_from_sock(char *s, size_t n, int fd) {
     static long charsleft = 0;
     static char inbuf[MS_LARGE_BUF*4];
@@ -512,9 +589,11 @@ char *get_line_from_sock(char *s, size_t n, int fd) {
 
     /* If there is no line in the buffer */
     while (charsleft == 0 || (cp = strchr(inbuf, '\n')) == NULL) {
-        FD_SET fdset = {1, fd};
+        FD_SET fdset;
         TIMEVAL tv = {3, 0}; /* 3 second timeout on reads */
         int nlen;
+        FD_ZERO(&fdset);
+        FD_SET(fd, &fdset);
         if (select(0, &fdset, NULL, NULL, &tv) == 0) {
             draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
                 "Metaserver timed out.");
@@ -551,6 +630,9 @@ char *get_line_from_sock(char *s, size_t n, int fd) {
 
 #endif /* Win32 */
 
+/**
+ *
+ */
 void *metaserver1_thread(void *junk)
 {
     struct protoent *protox;
@@ -732,7 +814,9 @@ void *metaserver1_thread(void *junk)
     return NULL;
 }
 
-
+/**
+ *
+ */
 int metaserver1_get_info(void) {
     pthread_t   thread_id;
     int	    ret;
@@ -788,13 +872,13 @@ int metaserver_check_status(void) {
     return status;
 }
 
-/* This contacts the metaserver and gets the list of servers.  returns 0
+/**
+ * This contacts the metaserver and gets the list of servers.  returns 0
  * on success, 1 on failure.  Errors will get dumped to stderr,
  * so most errors should be reasonably clear.
  * metaserver and meta_port are the server name and port number
  * to connect to.
  */
-
 int metaserver_get_info(char *metaserver, int meta_port) {
 
     meta_numservers = 0;
@@ -808,7 +892,8 @@ int metaserver_get_info(char *metaserver, int meta_port) {
     return 0;
 }
 
-/* Show the metaservers to the player.  We use draw_ext_info() to do
+/**
+ * Show the metaservers to the player.  We use draw_ext_info() to do
  * that, and also let the player know they can enter their own host name.
  */
 void metaserver_show(int show_selection) {
@@ -870,7 +955,8 @@ void metaserver_show(int show_selection) {
     pthread_mutex_unlock(&ms2_info_mutex);
 }
 
-/* String contains the selection that the player made for the metaserver.
+/**
+ * String contains the selection that the player made for the metaserver.
  * this may not be a a selection, but could be a host name or ip address.
  * this returns 0 on sucessful selection, 1 if failure (invalid selection
  * or the like.
@@ -959,41 +1045,13 @@ int metaserver_select(char *sel) {
         return 1;
     }
 
-    /* Add server to cache */
-    if ((num <= meta_numservers) && (num != meta_numservers + cached_servers_num + 1)) {
-        int index;
-        for (index = 0; index < cached_servers_num; index++) {
-            if (strcmp(server_name, cached_servers_name[index]) == 0)
-                break;
-        }
-        /* If server is first in cache, no need to re-add id */
-        if (index != 0 || !cached_servers_num) {
-            char *name;
-            char *ip;
-            int copy;
-
-            if (index == cached_servers_num) {
-                name = strdup(server_name);
-                ip = strdup(server_ip);
-                cached_servers_num++;
-                if (cached_servers_num > CACHED_SERVERS_MAX) {
-                    cached_servers_num--;
-                    free(cached_servers_name[cached_servers_num-1]);
-                    free(cached_servers_ip[cached_servers_num-1]);
-                }
-            } else {
-                name = cached_servers_name[index];
-                ip = cached_servers_ip[index];
-            }
-            for (copy = MIN(index, CACHED_SERVERS_MAX-1); copy > 0; copy--) {
-                cached_servers_name[copy] = cached_servers_name[copy-1];
-                cached_servers_ip[copy] = cached_servers_ip[copy-1];
-            }
-            cached_servers_name[0] = name;
-            cached_servers_ip[0] = ip;
-            metaserver_save_cache();
-        }
-    }
+    /*
+     * Upon successful connection, add the server to the cache or move it to
+     * the top of the list.
+     */
+    if ((num <= meta_numservers)
+    &&  (num != meta_numservers + cached_servers_num + 1))
+        metaserver_update_cache(server_name, server_ip); 
 
     return 0;
 }
@@ -1012,7 +1070,7 @@ int metaserver_select(char *sel) {
  * defined - trying to bring in the files the are defined
  * in just causes more dependencies, etc.
  */
-void draw_ext_info(int orig_color, int type, int subtype, char *message) {}
+void draw_ext_info(int orig_color, int type, int subtype, const char *message) {}
 int init_connection(char *host, int port) {}
 
 int metaserver2_on=1, metaserver_on=1;
@@ -1022,6 +1080,9 @@ ClientSocket csocket;
 char *meta_server=META_SERVER;
 int meta_port=META_PORT;
 
+/**
+ *
+ */
 void handle_ms_data(int msservernum) {
     int i;
 	fprintf(stderr,"Collecting data from metaserver %d.", msservernum);
@@ -1042,6 +1103,9 @@ void handle_ms_data(int msservernum) {
 	fprintf(stderr, "%d servers found\n", meta_numservers);
 }
 
+/**
+ *
+ */
 int main(int argc, char *argv[]) {
 
 #ifdef MS_SA_NOTMS2
