@@ -1,5 +1,5 @@
 const char * const rcsid_gtk2_config_c =
-    "$Id: config.c 12935 2010-04-22 07:14:54Z kbulgrien $";
+    "$Id: config.c 14240 2011-01-12 06:31:21Z mwedel $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -43,6 +43,48 @@ const char * const rcsid_gtk2_config_c =
 
 #include <dirent.h>
 
+#ifdef MINGW
+int alphasort(const struct dirent **a, const struct dirent **b)
+{
+    return strcoll((*a)->d_name, (*b)->d_name);
+}
+
+int scandir(const char *dir, struct dirent ***namelist,
+            int (*select)(const struct dirent *),
+            int (*compar)(const struct dirent **, const struct dirent **)) {
+    DIR *d;
+    struct dirent *entry;
+    register int i=0;
+    size_t entrysize;
+
+    if((d = opendir(dir)) == NULL)
+        return -1;
+
+    *namelist = NULL;
+    while ((entry = readdir(d)) != NULL)
+    {
+        if (select == NULL || (select != NULL && (*select)(entry)))
+        {
+            *namelist = (struct dirent **)realloc((void *)(*namelist),
+                                                  (size_t)((i + 1) * sizeof(struct dirent *)));
+            if (*namelist == NULL) return -1;
+            entrysize = sizeof(struct dirent) - sizeof(entry->d_name) + strlen(entry->d_name) + 1;
+            (*namelist)[i] = (struct dirent *)malloc(entrysize);
+            if ((*namelist)[i] == NULL) return -1;
+            memcpy((*namelist)[i], entry, entrysize);
+            i++;
+        }
+    }
+    if (closedir(d)) return -1;
+    if (i == 0) return -1;
+    if (compar != NULL)
+        qsort((void *)(*namelist), (size_t)i, sizeof(struct dirent *),
+              (int(*)(const void*, const void*))compar);
+    
+    return i;
+}
+#endif
+
 extern char window_xml_file[MAX_BUF];  /* Name of the .glade layout in use. */
 
 GtkWidget *config_window, *config_spinbutton_cwindow, *config_button_echo,
@@ -67,9 +109,9 @@ static const char * const display_modes[] = {"Pixmap", "SDL", "OpenGL"};
  * client theme if one is selected.  First, the player-specific rc files are
  * added to the GTK rc default files list.  ${HOME}/.crossfire/gtkrc is added
  * first.  All client sessions are affected by this rc file if it exists.
- * Next, ${HOME}/.crossfire/<layout>.gtkrc is added, where <layout> is the
+ * Next, ${HOME}/.crossfire/[layout].gtkrc is added, where [layout] is the
  * name of the .glade file that is loaded.  IE. If gtk-v2.glade is loaded,
- * <layout> is "gtk-v2".  This sets up the possibility for a player to make a
+ * [layout] is "gtk-v2".  This sets up the possibility for a player to make a
  * layout-specific rc file.  Finally, if the client theme is not "None", the
  * client theme file is added.  In most cases, the player-specific files are
  * probably not going to exist, so the theme system will continue to work the
@@ -84,82 +126,82 @@ static const char * const display_modes[] = {"Pixmap", "SDL", "OpenGL"};
  * is starting up, this is false, because all the widgets haven't been realized
  * yet, and the initialize routines will get the theme data at that time.
  */
-void load_theme(int reload)
+static char **default_files=NULL;
+void init_theme()
 {
-    static char **default_files=NULL;
     char path[MAX_BUF];
+    char xml_basename[MAX_BUF];
+    char **tmp;
+    char *cp;
     int i;
 
     /*
-     * We should only be called with reload once at startup.  at that time,
-     * store away the default rc files so we can restore them later.
+     * The GTK man page says copy of this data should be made, so do that.
      */
-    if (!reload) {
-        char xml_basename[MAX_BUF];
-        char **tmp;
-        char *cp;
-
-        /*
-         * The GTK man page says copy of this data should be made, so do that.
-         */
-        tmp = gtk_rc_get_default_files();
-        i = 0;
-        while (tmp[i]) {
-            i++;
-        }
-        /*
-         * Add two more GTK rc files that may be used by a player to customize
-         * the client appearance in general, or to customize the appearance
-         * of a specific .glade layout.  Allocate pointers to the local copy
-         * of the entire list.
-         */
-        i += 2;
-        default_files = malloc(sizeof(char*) * (i+1));
-        /*
-         * Copy in GTK's default list which probably contains system paths
-         * like <SYSCONFDIR>/gtk-2.0/gtkrc and user-specific files like
-         * ${HOME}/.gtkrc, or even LANGuage-specific ones like
-         * ${HOME}/.gtkrc.en, etc.
-         */
-        i=0;
-        while (tmp[i]) {
-            default_files[i] = strdup(tmp[i]);
-            i++;
-        }
-        /*
-         * Add a player-specific gtkrc to the list of default rc files.  This
-         * file is probably reserved for player use, though in all liklihood
-         * will not get used that much.  Still, it makes it easy for someone
-         * to make their own theme without having to have access to the
-         * system-wide theme folder.  This is the lowest priority client rc
-         * file as either a <layout>.gtkrc file or a client-configured theme
-         * settings can over-ride it.
-         */
-        snprintf(path, sizeof(path), "%s/.crossfire/gtkrc", getenv("HOME"));
-        default_files[i] = strdup(path);
+    tmp = gtk_rc_get_default_files();
+    i = 0;
+    while (tmp[i]) {
         i++;
-        /*
-         * Add a UI layout-specific rc file to the list of default list.  It
-         * seems reasonable to allow client code to have access to this file
-         * to make some basic changes to fonts, via a graphical interface.
-         * Truncate window_xml_file to remove a .extension if one exists, so
-         * that the window positions file can be created with a .gtkrc suffix.
-         * This is a mid-priority client rc file as its settings supersede the
-         * client gtkrc file, but are overridden by a client-configured theme.
-         */
-        strncpy(xml_basename, window_xml_file, MAX_BUF);
-        cp = strrchr(xml_basename, '.');
-        if (cp)
-          cp[0] = 0;
-        snprintf(path, sizeof(path),
-            "%s/.crossfire/%s.gtkrc", getenv("HOME"), xml_basename);
-        default_files[i] = strdup(path);
-        i++;
-        /*
-         * Mark the end of the list of default rc files.
-         */
-        default_files[i] = NULL;
     }
+    /*
+     * Add two more GTK rc files that may be used by a player to customize
+     * the client appearance in general, or to customize the appearance
+     * of a specific .glade layout.  Allocate pointers to the local copy
+     * of the entire list.
+     */
+    i += 2;
+    default_files = malloc(sizeof(char*) * (i+1));
+    /*
+     * Copy in GTK's default list which probably contains system paths
+     * like <SYSCONFDIR>/gtk-2.0/gtkrc and user-specific files like
+     * ${HOME}/.gtkrc, or even LANGuage-specific ones like
+     * ${HOME}/.gtkrc.en, etc.
+     */
+    i=0;
+    while (tmp[i]) {
+        default_files[i] = strdup(tmp[i]);
+        i++;
+    }
+    /*
+     * Add a player-specific gtkrc to the list of default rc files.  This
+     * file is probably reserved for player use, though in all liklihood
+     * will not get used that much.  Still, it makes it easy for someone
+     * to make their own theme without having to have access to the
+     * system-wide theme folder.  This is the lowest priority client rc
+     * file as either a <layout>.gtkrc file or a client-configured theme
+     * settings can over-ride it.
+     */
+    snprintf(path, sizeof(path), "%s/.crossfire/gtkrc", getenv("HOME"));
+    default_files[i] = strdup(path);
+    i++;
+    /*
+     * Add a UI layout-specific rc file to the list of default list.  It
+     * seems reasonable to allow client code to have access to this file
+     * to make some basic changes to fonts, via a graphical interface.
+     * Truncate window_xml_file to remove a .extension if one exists, so
+     * that the window positions file can be created with a .gtkrc suffix.
+     * This is a mid-priority client rc file as its settings supersede the
+     * client gtkrc file, but are overridden by a client-configured theme.
+     */
+    strncpy(xml_basename, window_xml_file, MAX_BUF);
+    cp = strrchr(xml_basename, '.');
+    if (cp)
+        cp[0] = 0;
+    snprintf(path, sizeof(path),
+             "%s/.crossfire/%s.gtkrc", getenv("HOME"), xml_basename);
+    CONVERT_FILESPEC_TO_OS_FORMAT(path);
+    default_files[i] = strdup(path);
+    i++;
+    /*
+     * Mark the end of the list of default rc files.
+     */
+    default_files[i] = NULL;
+}
+
+void load_theme(int reload) {
+    char path[MAX_BUF];
+    int i;
+
     /*
      * Whether or not this is default and initial run, we want to register
      * the modified rc search path list, so GTK needs to get the changes.
@@ -192,32 +234,30 @@ void load_theme(int reload)
         gtk_rc_add_default_file(path);
     }
 
-    if (reload) {
-        /*
-         * Require GTK to reparse and rebind all the widget data.
-         */
-        gtk_rc_reparse_all_for_settings(
-            gtk_settings_get_for_screen(gdk_screen_get_default()), TRUE);
-        gtk_rc_reset_styles(
-            gtk_settings_get_for_screen(gdk_screen_get_default()));
-        /*
-         * Call client functions to reparse the custom widgets it controls.
-         */
-        info_get_styles();
-        inventory_get_styles();
-        stats_get_styles();
-        spell_get_styles();
-        update_spell_information();
-        /*
-         * Set inv_updated to force a redraw - otherwise it will not
-         * necessarily bind the lists with the new widgets.
-         */
-        cpl.below->inv_updated = 1;
-        cpl.ob->inv_updated = 1;
-        draw_lists();
-        draw_stats(TRUE);
-        draw_message_window(TRUE);
-    }
+    /*
+     * Require GTK to reparse and rebind all the widget data.
+     */
+    gtk_rc_reparse_all_for_settings(
+                                    gtk_settings_get_for_screen(gdk_screen_get_default()), TRUE);
+    gtk_rc_reset_styles(
+                        gtk_settings_get_for_screen(gdk_screen_get_default()));
+    /*
+     * Call client functions to reparse the custom widgets it controls.
+     */
+    info_get_styles();
+    inventory_get_styles();
+    stats_get_styles();
+    spell_get_styles();
+    update_spell_information();
+    /*
+     * Set inv_updated to force a redraw - otherwise it will not
+     * necessarily bind the lists with the new widgets.
+     */
+    cpl.below->inv_updated = 1;
+    cpl.ob->inv_updated = 1;
+    draw_lists();
+    draw_stats(TRUE);
+    draw_message_window(TRUE);
 }
 
 /**
@@ -236,6 +276,7 @@ void load_defaults(void)
     }
 
     snprintf(path, sizeof(path), "%s/.crossfire/gdefaults2", getenv("HOME"));
+    CONVERT_FILESPEC_TO_OS_FORMAT(path);
     if ((fp=fopen(path,"r"))==NULL) return;
     while (fgets(inbuf, MAX_BUF-1, fp)) {
         inbuf[MAX_BUF-1]='\0';
@@ -389,6 +430,7 @@ void save_defaults(void)
     int i;
 
     snprintf(path, sizeof(path), "%s/.crossfire/gdefaults2", getenv("HOME"));
+    CONVERT_FILESPEC_TO_OS_FORMAT(path);
     if (make_path_to_file(path)==-1) {
         LOG(LOG_ERROR, "config.c::save_defaults","Could not create %s", path);
         return;
@@ -475,17 +517,20 @@ void config_init(GtkWidget *window_root)
     config_combobox_glade =
         glade_xml_get_widget(xml_tree, "config_combobox_glade");
 
+    g_signal_connect((gpointer) config_window, "delete_event",
+        G_CALLBACK(gtk_widget_hide_on_delete), NULL);
+
     widget = glade_xml_get_widget(xml_tree, "config_button_save");
-    g_signal_connect ((gpointer) widget, "clicked",
-        G_CALLBACK (on_config_button_save_clicked), NULL);
+    g_signal_connect((gpointer) widget, "clicked",
+        G_CALLBACK(on_config_button_save_clicked), NULL);
 
     widget = glade_xml_get_widget(xml_tree, "config_button_apply");
-    g_signal_connect ((gpointer) widget, "clicked",
-        G_CALLBACK (on_config_button_apply_clicked), NULL);
+    g_signal_connect((gpointer) widget, "clicked",
+        G_CALLBACK(on_config_button_apply_clicked), NULL);
 
     widget = glade_xml_get_widget(xml_tree, "config_button_close");
-    g_signal_connect ((gpointer) widget, "clicked",
-        G_CALLBACK (on_config_button_close_clicked), NULL);
+    g_signal_connect((gpointer) widget, "clicked",
+        G_CALLBACK(on_config_button_close_clicked), NULL);
 
     /*
      * Display mode combo box setup.  First, remove all entries, then populate
@@ -604,6 +649,8 @@ static void fill_combobox_from_datadir(GtkWidget *combobox, char *active,
         snprintf(path, MAX_BUF, "%s/%s", CF_DATADIR, subdir);
 
         count = scandir(path, &files, *scandir_filter, alphasort);
+        LOG(LOG_DEBUG, "config.c::fill_combobox_from_datadir",
+            "found %d files in %s\n", count, path);
 
         for (i=0; i<count; i++) {
             /*
@@ -1019,6 +1066,7 @@ void save_winpos(void)
       cp[0] = 0;
 
     snprintf(savename, sizeof(savename), "%s/.crossfire/%s.pos", getenv("HOME"), buf);
+    CONVERT_FILESPEC_TO_OS_FORMAT(savename);
     if (!(save = fopen(savename, "w"))) {
         snprintf(buf, sizeof(buf), "Cannot open %s - window positions not saved!", savename);
         draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_ERROR, buf);
@@ -1099,6 +1147,7 @@ void load_window_positions(GtkWidget *window_root)
       cp[0] = 0;
 
     snprintf(loadname, sizeof(loadname), "%s/.crossfire/%s.pos", getenv("HOME"), buf);
+    CONVERT_FILESPEC_TO_OS_FORMAT(loadname);
     if (!(load = fopen(loadname, "r"))) {
         snprintf(buf, sizeof(buf),
             "Cannot open %s: Using default window positions.", loadname);
