@@ -1,5 +1,5 @@
 const char * const rcsid_gtk2_pickup_c =
-    "$Id: pickup.c 12875 2010-04-09 07:04:28Z mwedel $";
+    "$Id: pickup.c 13958 2010-10-02 09:34:23Z ryo_saeba $";
 
 /*
     Crossfire client, a client program for the crossfire program.
@@ -95,6 +95,7 @@ static int num_pickups=0;
 #define PU_NOT_CURSED           0x01000000
 #define PU_JEWELS               0x02000000
 #define PU_FLESH                0x04000000
+#define PU_CONTAINERS           0x08000000
 
 static unsigned int pmode=0, no_recurse=0;
 
@@ -277,6 +278,13 @@ on_menu_jewels_activate                  (GtkMenuItem     *menuitem,
     new_menu_pickup(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)), PU_JEWELS);
 }
 
+void
+on_menu_containers_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    new_menu_pickup(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)), PU_CONTAINERS);
+}
+
 void
 on_menu_flesh_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -530,6 +538,11 @@ void pickup_init(GtkWidget *window_root)
     pickup_mapping[num_pickups].pickup_mode = PU_JEWELS;
     num_pickups++;
 
+    pickup_mapping[num_pickups].menuitem =
+        glade_xml_get_widget(xml_tree, "containers");
+    pickup_mapping[num_pickups].pickup_mode = PU_CONTAINERS;
+    num_pickups++;
+
     pickup_mapping[num_pickups].menuitem =
         glade_xml_get_widget(xml_tree, "all_weapons");
     pickup_mapping[num_pickups].pickup_mode = PU_ALLWEAPON;
