@@ -1,5 +1,5 @@
 const char * const rcsid_gtk2_png_c =
-    "$Id: png.c 9193 2008-06-01 14:26:32Z anmaster $";
+    "$Id: png.c 13981 2010-10-09 21:43:20Z anmaster $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -201,7 +201,7 @@ uint8 *png_to_data(uint8 *data, int len, uint32 *width, uint32 *height)
 
     if (!pixels) {
         png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
-        LOG(LOG_CRITICAL,"gtk::png_to_data","Out of memory - exiting");
+        LOG(LOG_CRITICAL,"gtk-v2::png_to_data","Out of memory - exiting");
         exit(1);
     }
 
@@ -281,7 +281,7 @@ uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
     if (*width > MAX_IMAGE_WIDTH || new_width > MAX_IMAGE_WIDTH
     || *height > MAX_IMAGE_HEIGHT || new_height > MAX_IMAGE_HEIGHT)
     {
-        LOG(LOG_CRITICAL,"gtk::rescale_rgba_data","Image too big");
+        LOG(LOG_CRITICAL,"gtk-v2::rescale_rgba_data","Image too big");
         exit(0);
     }
 
@@ -547,7 +547,7 @@ int png_to_gdkpixmap(GdkWindow *window, uint8 *data, int len,
     static uint8 *pixels=NULL;
     static int pixels_byte=0, rows_byte=0;
     static png_bytepp   rows=NULL;
-    unsigned long width, height;
+    png_uint_32 width, height;
     png_structp png_ptr;
     png_infop   info_ptr;
     int bit_depth, color_type, interlace_type, compression_type, filter_type,
